#!/usr/bin/env python
# vim: ft=python
# TODO support py2 if requests_html supports
__doc__ = "Simple CLI for searching word meanings from UrbanDictionary"


import urllib

import requests_html


def remove_dup(xs):
    return list({i: None for i in xs}.keys())


def cambridge_fr(word, dictionary="french-english"):
    url = "https://dictionary.cambridge.org/dictionary/{}/{}".format(
        dictionary, urllib.parse.quote(word)
    )

    sess = requests_html.HTMLSession()
    resp = sess.get(url)

    ipa_nodes = resp.html.xpath('//span[@class="ipa dipa"]')
    ipa = []
    for i in ipa_nodes:
        ipa.append(i.text)
    ms = resp.html.xpath('//span[@class="trans dtrans"]')
    return {"url": url, "ipa": remove_dup(ipa), "means": remove_dup(m.text for m in ms)}


def cambridge(word, dictionary="english"):

    url = "https://dictionary.cambridge.org/dictionary/{}/{}".format(
        dictionary, urllib.parse.quote(word)
    )

    sess = requests_html.HTMLSession()
    resp = sess.get(url)

    ipa_nodes = resp.html.xpath('//span[@class="ipa dipa lpr-2 lpl-1"]')
    ipa = []
    for i in ipa_nodes:
        ipa.append(i.text)

    ms = resp.html.xpath('//div[@class="def ddef_d db"]')
    return {"url": url, "ipa": remove_dup(ipa), "means": remove_dup(m.text for m in ms)}


def urbandictionary(word):
    """
    >>> "misleading" in " ".join(urbandictionary("red herring")["means"])
    True
    >>> "that feel when" in " ".join(urbandictionary("tfw")["means"]).lower()
    True
    """
    sess = requests_html.HTMLSession()
    url = "https://www.urbandictionary.com/define.php?term={}".format(
        urllib.parse.quote(word)
    )
    r = sess.get(url)
    meaning_divs = r.html.xpath('//div[@class="meaning"]')
    if not meaning_divs:
        if "There are no definitions for this word" in r.html.full_text:
            return url, ["There are no definitions for this word"]
        else:
            raise Exception("Unknown result for {}: {}".format(word, r.html.text))

    return {
        "url": url,
        "ipa": "",
        "means": [node.text for node in meaning_divs],
    }


def get_meanings(word, source="urban"):
    if source == "urban":
        return urbandictionary(word)
    elif source == "fr":
        return cambridge_fr(word)
    else:
        return cambridge(word)


def _test():
    import doctest

    doctest.testmod()



import argparse
import html
import crayons
import sys
import textwrap


import uds


def colorize(text, color, bold=False):
    """
    https://github.com/kennethreitz/crayons
    Included colors are red, green, yellow, blue, black,
    magenta, cyan, white, and normal
    """
    return eval("str(crayons.{}(text, bold=bold))".format(color))


def main():
    parser = argparse.ArgumentParser(__doc__)
    parser.add_argument(
        "word",
        help="Word to search for meaning on UrbanDictionary",
        type=str,
        nargs="+",
    )
    parser.add_argument(
        "-s", "--source", help="Which dictionary to search", type=str, default="urban",
    )
    parser.add_argument(
        "-n", "--count", help="Number of meanings to display", type=int, default=8,
    )
    args = parser.parse_args()
    word_to_search = " ".join(args.word)

    def normalize(text, padding=" " * 6):
        # NOTE unescape seems do not work as text missing `;`, e.g `&apos;` ->
        # `&apos`, thus, use dirty replace
        cleaned = html.unescape(text).replace("&apos", "'")
        cleaned = textwrap.fill(cleaned, width=120)
        cleaned = textwrap.indent(cleaned, prefix=padding).strip()
        return cleaned

    def pad_lines(text, pad=""):
        padded = "\n        ".join(
            (
                "{pad}{line}".format(line=normalize(line), pad=pad)
                for line in text.splitlines()
                if line
            )
        )
        return padded

    if args.source == "urban":
        dictionary = "UrbanDictionary"
    elif "cam" in args.source:
        dictionary = "Cambridge"
    elif "fr" in args.source:
        dictionary = "fr"

    print(
        colorize(
            "{} results for {}".format(
                dictionary, colorize(word_to_search, "yellow", bold=True)
            ),
            "blue",
            bold=True,
        )
    )

    result = uds.get_meanings(word_to_search, source=args.source)
    meanings = result["means"]
    ipa = result["ipa"]
    if not meanings:
        sys.exit(crayons.red("There are no definitions for this word."))

    print(ipa)
    for idx, _meaning in enumerate(meanings, start=1):
        result = "{:>2}{idx}.  {meaning}".format(
            "", idx=colorize(idx, "red"), meaning=pad_lines(_meaning)
        )
        print(result)
        if idx == args.count:
            break


if __name__ == "__main__":
    main()
